package se.su.dsv.scipro.checklist;

import se.su.dsv.scipro.project.Project;
import se.su.dsv.scipro.system.AbstractServiceImpl;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;

public class ChecklistServiceImpl extends AbstractServiceImpl<Checklist, Long> implements ChecklistService {
    @Inject
    public ChecklistServiceImpl(Provider<EntityManager> em) {
        super(em, Checklist.class, QChecklist.checklist);
    }

    @Override
    public Long countAnswers(Project project, ChecklistAnswerEnum answer) {
        return from(QChecklist.checklist)
                .join(QChecklist.checklist.questions, QChecklistQuestion.checklistQuestion)
                .join(QChecklistQuestion.checklistQuestion.answers, QChecklistAnswer.checklistAnswer)
                .where(
                        QChecklist.checklist.project.eq(project),
                        QChecklistAnswer.checklistAnswer.answer.eq(answer))
                .count();
    }

    @Override
    public Long countUnanswered(Project project) {
        long questions = from(QChecklist.checklist)
                .join(QChecklist.checklist.questions)
                .where(QChecklist.checklist.project.eq(project))
                .count();
        questions = questions * project.getProjectParticipants().size();

        long answers = from(QChecklist.checklist)
                .join(QChecklist.checklist.questions, QChecklistQuestion.checklistQuestion)
                .join(QChecklistQuestion.checklistQuestion.answers, QChecklistAnswer.checklistAnswer)
                .where(QChecklist.checklist.project.eq(project))
                .count();

        return questions - answers;
    }
}