package se.su.dsv.scipro.peer;

import lombok.AccessLevel;
import lombok.Getter;
import lombok.Setter;
import lombok.ToString;
import se.su.dsv.scipro.system.DomainObject;
import se.su.dsv.scipro.system.User;

import javax.persistence.*;

@Getter
@Setter
@ToString(exclude = {"commentThread"})
@Entity
@Table(name = "comment")
@Cacheable(true)
@SuppressWarnings({"PMD.UnusedPrivateField", "PMD.SingularField"})
public class Comment extends DomainObject implements Comparable<Comment> {
    @Id
    @GeneratedValue
    @Setter(AccessLevel.NONE)
    private Long id;

    @ManyToOne(optional = false)
    private User creator;

    @Lob
    private String comment;

    @ManyToOne(optional = false)
    private CommentThread commentThread;

    protected Comment() {
    }

    public Comment(final User creator, final CommentThread commentThread) {
        this.creator = creator;
        this.commentThread = commentThread;
    }

    @Override
    public int compareTo(Comment o) {
        if (this.equals(o)) {
            return 0;
        }
        if (this.getDateCreated() != null && o.getDateCreated() != null) {
            if (o.getDateCreated().equals(this.getDateCreated())) {
                return o.getId().compareTo(this.getId());
            }
            return this.getDateCreated().compareTo(o.getDateCreated());
        }
        if (this.getId() != null && o.getId() != null) {
            return o.getId().compareTo(this.getId());
        }
        if (this.comment != null && o.comment != null) {
            return o.comment.compareTo(comment);
        }
        return 0;
    }
}
