package se.su.dsv.scipro.reviewing;

import com.google.common.eventbus.EventBus;
import scala.Option;
import scala.runtime.AbstractFunction0;
import scala.runtime.AbstractFunction1;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.file.FileDescriptionService;
import se.su.dsv.scipro.file.FileUpload;
import se.su.dsv.scipro.project.Project;

import javax.inject.Inject;
import javax.inject.Provider;
import javax.persistence.EntityManager;

class RoughDraftApprovalServiceImpl extends AbstractReviewerApprovalServiceImpl<RoughDraftApproval> implements RoughDraftApprovalService {

    @Inject
    RoughDraftApprovalServiceImpl(Provider<EntityManager> em, EventBus eventBus, FileDescriptionService fileDescriptionService) {
        super(em, RoughDraftApproval.class, QRoughDraftApproval.roughDraftApproval, eventBus, fileDescriptionService);
    }

    @Override
    public RoughDraftApproval requestApproval(final Project project, FileUpload thesis, final String comment) {
        final FileDescription storedThesis = fileDescriptionService.storeFile(thesis);
        final RoughDraftApproval roughDraftApproval = findBy(project).map(new AbstractFunction1<RoughDraftApproval, RoughDraftApproval>() {
            @Override
            public RoughDraftApproval apply(final RoughDraftApproval approval) {
                approval.addNewThesis(storedThesis, comment);
                return approval;
            }
        }).getOrElse(new AbstractFunction0<RoughDraftApproval>() {
            @Override
            public RoughDraftApproval apply() {
                return newApproval(project, storedThesis, comment);
            }
        });
        final RoughDraftApproval persisted = save(roughDraftApproval);
        eventBus.post(new RoughDraftApprovalRequestedEvent(persisted));
        return persisted;
    }

    @Override
    protected RoughDraftApproval newApproval(Project project, FileDescription storedThesis, String comment) {
        return new RoughDraftApproval(project, storedThesis, comment);
    }

    @Override
    public Option<RoughDraftApproval> findBy(Project project) {
        return Option.apply(findOne(QRoughDraftApproval.roughDraftApproval.project.eq(project)));
    }

}
