package se.su.dsv.scipro.reviewing;

import org.joda.time.Instant;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.misc.DaysService;

import java.util.Date;

import static org.joda.time.Instant.now;
import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

@RunWith(MockitoJUnitRunner.class)
public class DecisionTest {
    @Mock
    private DaysService daysService;

    @Test
    public void calculates_deadline_using_days_service() throws Exception {
        Date deadline = new Date(46345235432L);
        Instant requested = now();

        when(daysService.workDaysAfter(requested.toDate(), 8)).thenReturn(deadline);

        Decision decision = createDecisionRequestedAt(requested);
        Instant calculatedDeadline = decision.getDeadline(daysService, 8);

        assertEquals(deadline, calculatedDeadline.toDate());
    }

    private Decision createDecisionRequestedAt(final Instant requested) {
        return new Decision(mock(ReviewerApproval.class), mock(FileDescription.class), requested, "test");
    }
}