package se.su.dsv.scipro.integration.daisy.workers;

import com.google.common.eventbus.EventBus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import scala.Option;
import se.su.dsv.scipro.grading.GradingCompletedEvent;
import se.su.dsv.scipro.io.dto.Course;
import se.su.dsv.scipro.io.dto.StudentProjectParticipant;
import se.su.dsv.scipro.io.http.DaisyAPI;
import se.su.dsv.scipro.project.*;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.system.UserService;
import se.su.dsv.scipro.workerthreads.AbstractWorker;

import javax.inject.Inject;
import javax.inject.Named;
import java.util.Set;

@Named
public class GradingCompletedMilestoneActivator extends AbstractWorker{
    private static final Logger LOG = LoggerFactory.getLogger(GradingCompletedMilestoneActivator.class);

    private final ProjectService projectService;
    private final DaisyAPI daisyAPI;
    private final EventBus eventBus;
    private final UserService userService;


    @Inject
    public GradingCompletedMilestoneActivator(ProjectService projectRepo, DaisyAPI daisyAPI, EventBus eventBus, UserService userService) {
        this.projectService = projectRepo;
        this.daisyAPI = daisyAPI;
        this.eventBus = eventBus;
        this.userService = userService;
    }

    @Override
    protected void doWork() {
        beginTransaction();
        Iterable<Project> projects = projectService.findAll(QProject.project.projectStatus.eq(ProjectStatus.ACTIVE));

        User user;

        for (Project project : projects) {
            try {
                Set<StudentProjectParticipant> participatingAuthors = daisyAPI.getAuthors(project.getIdentifier());
                for (StudentProjectParticipant participatingAuthor: participatingAuthors) {
                    Course course  = participatingAuthor.getCourse();
                    Option<String> grade = daisyAPI.getGradeForStudentInCourse(participatingAuthor.getId(), course.getId());
                    if (grade.isDefined()) {
                        user = userService.findByExternalIdentifier(participatingAuthor.getPerson().getId());
                        eventBus.post(new GradingCompletedEvent(project, user));
                    }
                }
            }
            catch (Exception e) {
                LOG.info("Failed to make 'grading completed' milestone as completed for project ({})", project, e);
            }
        }
        commitTransaction();
    }
}
