package se.su.dsv.scipro.loginlogout.pages;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.markup.html.form.*;
import org.apache.wicket.markup.html.link.BookmarkablePageLink;
import org.apache.wicket.markup.html.panel.FeedbackPanel;
import org.apache.wicket.model.Model;
import se.su.dsv.scipro.basepages.PublicPage;
import se.su.dsv.scipro.security.auth.roles.Roles;
import se.su.dsv.scipro.system.*;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.HashSet;

public class FirstTimeRegistrationPage extends PublicPage {

    public static final String USER_NAME = "userName";
    public static final String FORM = "form";
    public static final String PASSWORD = "password";
    public static final String FEEDBACK = "feedback";
    public static final String BACK = "back";
    private final TextField<String> nameField;
    private final TextField<String> passwordField;

    @Inject
    private UserService userService;
    @Inject
    private UserNameService usernameService;
    @Inject
    private PasswordRepo passwordRepo;

    public FirstTimeRegistrationPage() {
        super();

        if (userService.count() > 0) {
            throw new RestartResponseException(getApplication().getHomePage());
        }

        add(new FeedbackPanel(FEEDBACK));

        Form<Void> form = new StatelessForm<Void>(FORM) {
            @Override
            protected void onSubmit() {
                User user = createUser();
                Username username = createUsernameObject(user);
                user.setUsernames(new HashSet<>(Arrays.asList(username)));
                user.setEmailAddress(nameField.getModelObject());
                user.addRole(Roles.SYSADMIN);
                Password password = createPassword(user, passwordField.getModelObject());
                user = userService.save(user);
                password = passwordRepo.save(password);
                user.setPassword(password);
                userService.save(user);
                usernameService.save(username);
                success(getString("success"));
            }
        };

        nameField = new RequiredTextField<>(USER_NAME, new Model<String>());
        form.add(nameField);

        passwordField = new PasswordTextField(PASSWORD, new Model<String>());
        form.add(passwordField);

        add(form);

        add(new BookmarkablePageLink<Void>(BACK, getApplication().getHomePage()));
    }

    private User createUser() {
        return new User();
    }

    private Username createUsernameObject(final User user) {
        Username username = new Username();
        username.setUsername(nameField.getModelObject());
        username.setUser(user);

        return username;
    }

    public Password createPassword(final User user, final String passwordString) {
        char[] passwordInput = passwordString.toCharArray();

        Password password = new Password();
        password.setSalt(PasswordHandler.generateSalt());
        password.setHash(PasswordHandler.generateHash(passwordInput, password.getSalt()));
        password.setUser(user);

        return password;
    }
}
