package se.su.dsv.scipro.peer;

import org.apache.wicket.RestartResponseException;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.request.mapper.parameter.PageParameters;
import se.su.dsv.scipro.components.menuhighlighting.MenuHighlightAuthorMyProjects;
import se.su.dsv.scipro.data.DetachableServiceModel;
import se.su.dsv.scipro.data.dataobjects.Member;
import se.su.dsv.scipro.generalsystemsettings.GeneralSystemSettingsService;
import se.su.dsv.scipro.panels.CommentThreadPanel;
import se.su.dsv.scipro.profile.UserLinkPanel;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;
import se.su.dsv.scipro.util.PageParameterKeys;

import javax.inject.Inject;

public class ViewPeerReviewPage extends AbstractPeerPortalPage implements MenuHighlightAuthorMyProjects {

    @Inject
    private PeerReviewService peerReviewService;
    @Inject
    private GeneralSystemSettingsService generalSystemSettingsService;

    public ViewPeerReviewPage(PageParameters pp) {
        super(pp);

        add(new PeerPortalHeader("header", projectModel) {
            @Override
            protected boolean submissionStartsCollapsed() {
                return true;
            }
        });

        long reviewId = pp.get(PageParameterKeys.MAP.get(PeerReview.class)).toLong();
        IModel<PeerReview> reviewModel = new DetachableServiceModel<>(peerReviewService, reviewId);

        if (!checkAccess(SciProSession.get().getUser(), reviewModel.getObject())) {
            throw new RestartResponseException(getApplication().getHomePage());
        }

        add(new DisplayReviewPanel("review", reviewModel));
        add(new ReviewPageRequestDetailsPanel("request", new PropertyModel<PeerRequest>(reviewModel, "peerRequest")));
        add(new UserLinkPanel("reviewedBy", new PropertyModel<User>(reviewModel, "reviewer")));
        add(new CommentThreadPanel("comment", reviewModel));
    }

    private boolean checkAccess(User user, PeerReview peerReview) {
        if (generalSystemSettingsService.getGeneralSystemSettingsInstance().isPublicReviewsActivated()) {
            return true;
        }

        for (Member member : peerReview.getMembers()) {
            if (user.equals(member.getUser())) {
                return true;
            }
        }
        return false;
    }
}
