package se.su.dsv.scipro.reviewer;


import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.ajax.markup.html.form.AjaxCheckBox;
import org.apache.wicket.extensions.markup.html.repeater.data.grid.ICellPopulator;
import org.apache.wicket.extensions.markup.html.repeater.data.table.AbstractColumn;
import org.apache.wicket.extensions.markup.html.repeater.data.table.IColumn;
import org.apache.wicket.extensions.markup.html.repeater.util.SortableDataProvider;
import org.apache.wicket.markup.html.link.AbstractLink;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.springframework.data.domain.PageRequest;
import scala.runtime.AbstractFunction1;
import se.su.dsv.scipro.components.*;
import se.su.dsv.scipro.components.datatables.MultipleUsersColumn;
import se.su.dsv.scipro.components.datatables.UserColumn;
import se.su.dsv.scipro.data.enums.DateStyle;
import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.reviewing.MyReviewService;
import se.su.dsv.scipro.reviewing.ReviewerApproval;
import se.su.dsv.scipro.reviewing.ReviewerDeadlineSettingsService;
import se.su.dsv.scipro.reviewing.Status;
import se.su.dsv.scipro.session.SciProSession;
import se.su.dsv.scipro.system.User;

import javax.inject.Inject;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

public abstract class ApprovalReviewerPanel extends Panel {

    @Inject
    private MyReviewService myReviewService;
    @Inject
    private ReviewerDeadlineSettingsService reviewerDeadlineSettingsService;
    @Inject
    private DaysService daysService;

    public ApprovalReviewerPanel(final String id) {
        super(id);
        final IModel<MyReviewService.Filter> filter = getFilter();
        final DefaultDataPanel<ReviewerApproval, String> table = new DefaultDataPanel<>("undecidedApprovals", createColumns(), new UndecidedFinalSeminarApprovalsProvider(filter));
        table.setOutputMarkupId(true);
        add(table);
        add(new AjaxCheckBox("showAll", new PropertyModel<Boolean>(filter, "showAll")) {
            @Override
            protected void onUpdate(AjaxRequestTarget target) {
                target.add(table);
            }
        });
    }

    private IModel<MyReviewService.Filter> getFilter() {
        MyReviewService.Filter filter = new MyReviewService.Filter();
        filter.setUser(SciProSession.get().getUser());
        filter.setStep(getStep());
        return Model.of(filter);
    }

    private List<? extends IColumn<ReviewerApproval, String>> createColumns() {
        return Arrays.asList(
                new DateColumn<ReviewerApproval>(Model.of("Requested"), "dateCreated"),
                new AbstractColumn<ReviewerApproval, String>(Model.of("Deadline")) {
                    @Override
                    public void populateItem(Item<ICellPopulator<ReviewerApproval>> cellItem, String componentId, IModel<ReviewerApproval> rowModel) {
                        cellItem.add(new DateLabel(componentId, Model.of(rowModel.getObject().getCurrentDeadline(daysService, reviewerDeadlineSettingsService.getInstance())), DateStyle.DATE));
                    }
                },
                new AbstractColumn<ReviewerApproval, String>(Model.of("Title"), "project.title") {
                    @Override
                    public void populateItem(final Item<ICellPopulator<ReviewerApproval>> cellItem, final String componentId, final IModel<ReviewerApproval> rowModel) {
                        cellItem.add(LinkWrapper.apply(componentId, new AbstractFunction1<String, AbstractLink>() {
                            @Override
                            public AbstractLink apply(final String id) {
                                AbstractLink link = newDecisionLink(id, rowModel);
                                link.setBody(new PropertyModel<>(rowModel, "project.title"));
                                return link;
                            }
                        }));
                    }
                },
                new EnumPropertyColumn<ReviewerApproval, Status>(Model.of("Current status"), "currentStatus"),
                new MultipleUsersColumn<ReviewerApproval, String>(Model.of("Authors")) {
                    @Override
                    public IModel<List<User>> getUsers(final IModel<ReviewerApproval> rowModel) {
                        return new ListAdapterModel<>(new PropertyModel<Iterable<User>>(rowModel, "project.projectParticipants"));
                    }
                },
                new UserColumn<ReviewerApproval, String>(Model.of("Head supervisor")) {
                    @Override
                    protected IModel<User> getUser(final IModel<ReviewerApproval> rowModel) {
                        return new PropertyModel<>(rowModel, "project.headSupervisor");
                    }
                }
        );
    }

    protected abstract AbstractLink newDecisionLink(String id, IModel<ReviewerApproval> rowModel);

    public abstract ReviewerApproval.Step getStep();

    private final class UndecidedFinalSeminarApprovalsProvider extends SortableDataProvider<ReviewerApproval, String> {
        private IModel<MyReviewService.Filter> filter;

        private UndecidedFinalSeminarApprovalsProvider(IModel<MyReviewService.Filter> filter) {
            this.filter = filter;
        }

        @Override
        public Iterator<ReviewerApproval> iterator(final long first, final long count) {
            return myReviewService.findAll(filter.getObject(), new PageRequest(first, count)).iterator();
        }

        @Override
        public long size() {
            return myReviewService.count(filter.getObject());
        }

        @Override
        public IModel<ReviewerApproval> model(final ReviewerApproval object) {
            final Long id = object.getId();
            return new LoadableDetachableModel<ReviewerApproval>() {
                @Override
                protected ReviewerApproval load() {
                    return myReviewService.findApproval(id);
                }
            };
        }
    }

}
