package se.su.dsv.scipro.statistics;

import org.apache.wicket.markup.head.IHeaderResponse;
import org.apache.wicket.markup.head.JavaScriptHeaderItem;
import org.apache.wicket.markup.html.basic.EnumLabel;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.markup.html.list.PropertyListView;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.LoadableDetachableModel;
import org.apache.wicket.model.PropertyModel;
import se.su.dsv.scipro.components.DateLabel;
import se.su.dsv.scipro.misc.DaysService;
import se.su.dsv.scipro.reviewing.*;
import se.su.dsv.scipro.util.Pair;

import javax.inject.Inject;
import java.util.Date;
import java.util.List;

public class AdminReviewerDeadlinesPage extends AbstractAdminStatisticsPage {
    @Inject
    private ReviewerDeadlineFollowupService reviewerDeadlineFollowupService;
    @Inject
    private ReviewerDeadlineSettingsService reviewerDeadlineSettingsService;
    @Inject
    private DaysService daysService;

    public AdminReviewerDeadlinesPage() {
        add(new PropertyListView<Pair<RoughDraftApproval, Decision>>("roughDrafts", lateRoughDrafts()) {
            @Override
            protected void populateItem(final ListItem<Pair<RoughDraftApproval, Decision>> item) {
                item.add(new Label("head.project.title"));
                item.add(new Label("head.project.reviewer.fullName"));
                final int daysToDecide = reviewerDeadlineSettingsService.getInstance().getRoughDraftApproval();
                IModel<Date> deadline = new DeadlineModel(new PropertyModel<Decision>(item.getModel(), "tail"), daysToDecide);
                item.add(new DateLabel("deadline", deadline));
                item.add(new DateLabel("tail.requested"));
                item.add(new EnumLabel<Status>("tail.status"));
            }
        });
        add(new PropertyListView<Pair<FinalSeminarApproval, Decision>>("finalSeminarApprovals", lateFinalSeminarApprovals()) {
            @Override
            protected void populateItem(final ListItem<Pair<FinalSeminarApproval, Decision>> item) {
                item.add(new Label("head.project.title"));
                item.add(new Label("head.project.reviewer.fullName"));
                final int daysToDecide = reviewerDeadlineSettingsService.getInstance().getFinalSeminarApproval();
                IModel<Date> deadline = new DeadlineModel(new PropertyModel<Decision>(item.getModel(), "tail"), daysToDecide);
                item.add(new DateLabel("deadline", deadline));
                item.add(new DateLabel("tail.requested"));
                item.add(new EnumLabel<Status>("tail.status"));
            }
        });
    }

    private IModel<? extends List<? extends Pair<FinalSeminarApproval, Decision>>> lateFinalSeminarApprovals() {
        return new LoadableDetachableModel<List<Pair<FinalSeminarApproval, Decision>>>() {
            @Override
            protected List<Pair<FinalSeminarApproval, Decision>> load() {
                return reviewerDeadlineFollowupService.getLateFinalSeminarApprovals();
            }
        };
    }

    private IModel<? extends List<? extends Pair<RoughDraftApproval, Decision>>> lateRoughDrafts() {
        return new LoadableDetachableModel<List<Pair<RoughDraftApproval, Decision>>>() {
            @Override
            protected List<Pair<RoughDraftApproval, Decision>> load() {
                return reviewerDeadlineFollowupService.getLateRoughDrafts();
            }
        };
    }

    private final class DeadlineModel extends LoadableDetachableModel<Date> {
        private final int daysToDecide;
        private final IModel<Decision> decision;

        private DeadlineModel(final IModel<Decision> decision, final int daysToDecide) {
            this.daysToDecide = daysToDecide;
            this.decision = decision;
        }

        @Override
        protected Date load() {
            return decision.getObject().getDeadline(daysService, daysToDecide).toDate();
        }
    }

    @Override
    public void renderHead(IHeaderResponse response) {
        super.renderHead(response);
        response.render(JavaScriptHeaderItem.forUrl("//cdn.datatables.net/1.10.0/js/jquery.dataTables.js"));
    }
}
